#!/bin/bash

classify_label() {
  local OUTPUT="$1"
  if grep -q "^Verification result: UNSAFE" $OUTPUT
  then
    FINAL=FALSE
  elif grep -q "^Verification result: SAFE" $OUTPUT
  then
    FINAL=TRUE
  else
    FINAL=UNKNOWN
  fi

  if grep -q -e "Invalid write found" -e "Invalid read found" -e "Invalid free found" $OUTPUT
  then
    FINAL=UNKNOWN
  fi

  echo "$FINAL"
}

classify_memory() {
  local OUTPUT="$1"
  if grep -q "^Verification result: UNSAFE" $OUTPUT
  then
    FINAL=FALSE
  elif grep -q "^Verification result: SAFE" $OUTPUT
  then
    FINAL=TRUE
  else
    FINAL=UNKNOWN
  fi

  if grep -q -e "Invalid write found" -e "Invalid read found" $OUTPUT && [ "$FINAL" == "FALSE" ]
  then
    echo "FALSE(valid-deref)"
    return
  elif grep -q -e "Invalid free found" $OUTPUT && [ "$FINAL" == "FALSE" ]
  then
    echo "FALSE(valid-free)"
    return
  elif grep -q -e "Memory leak found" $OUTPUT && [ "$FINAL" == "FALSE" ]
  then
    echo "FALSE(valid-memtrack)"
    return
  elif [ "$FINAL" == "FALSE" ]
  then
    FINAL="UNKNOWN"
  fi

  echo "$FINAL"
}

TESTCASE="$1"
if [ ! -f "$TESTCASE" ]
then
  echo "Usage: $0 TESTCASE [-64]" >&2
  exit 1
fi

shift

PROPFILE_TC="$TESTCASE.prp"
PROPFILE_DIR="$(dirname $TESTCASE)"/ALL.prp

if [ -f "$PROPFILE_TC" ]
then
  PROPFILE="$PROPFILE_TC"
elif [ -f "$PROPFILE_DIR" ]
then
  PROPFILE="$PROPFILE_DIR"
else
  echo "Cannot find property specification." >&2
  echo "These were the searched locations: " >&2
  echo "  $PROPFILE_TC" >&2
  echo "  $PROPFILE_DIR" >&2
  exit 1
fi

OUTPUT="$(mktemp)"
if grep -q 'label(ERROR)' $PROPFILE
then
  scripts/cpa.sh -CPAlien-label -preprocess $TESTCASE &>$OUTPUT
  classify_label $OUTPUT
elif grep -q 'LTL(G valid-.*)' $PROPFILE
then
  scripts/cpa.sh -CPAlien -preprocess $TESTCASE &>$OUTPUT
  classify_memory $OUTPUT
else
  echo "Weird property file?" >&2
  exit 1
fi


rm -f $OUTPUT
